﻿using C1.WPF.ExpressionEditor;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace C1ExpressionEditorSample
{
    /// <summary>
    /// Interaction logic for DataGridIntegration.xaml
    /// </summary>
    public partial class DataGridIntegration : UserControl
    {
        public DataGridIntegration()
        {
            InitializeComponent();

            List<Product> items = Product.GetData(200);
            CollectionViewSource view = new CollectionViewSource() { Source = items };
            grid.ItemsSource = view.View;

            C1ExpressionEditor c1ExpressionEditor1 = new C1ExpressionEditor();
            C1ExpressionEditor c1ExpressionEditor2 = new C1ExpressionEditor();
            c1ExpressionEditor1.Expression = "[Price]*0.95";
            c1ExpressionEditor2.Expression = "[Price]*0.8";
            grid.ExpressionEditors.Add("CustomField1", c1ExpressionEditor1);
            grid.ExpressionEditors.Add("CustomField2", c1ExpressionEditor2);
            comboBox.SelectedIndex = 0;
            editor.OkClick += Editor_OkClick;
            editor.CancelClick += Editor_CancelClick;
            grid.Loaded += Grid_Loaded;
        }

        private void Grid_Loaded(object sender, RoutedEventArgs e)
        {
            foreach (DataGridColumn column in grid.Columns)
            {
                if (column.Header.Equals("CustomField1"))
                {
                    // don't allow cell edits
                    column.IsReadOnly = true;
                    // use human-friendly header
                    column.Header = "Special Price (New Customers)";

                }
                if (column.Header.Equals("CustomField2"))
                {
                    // don't allow cell edits
                    column.IsReadOnly = true;
                    // use human-friendly header
                    column.Header = "Special Price (Card Holders)";
                }
            }
        }

        private void Editor_CancelClick(object sender, EventArgs e)
        {
            var field = ((ComboBoxItem)comboBox.SelectedItem).Tag as string;
            if (grid != null && grid.ExpressionEditors.Contains(field))
            {
                editor.Expression = grid.ExpressionEditors[field].Expression;
            }
        }

        private void Editor_OkClick(object sender, EventArgs e)
        {
            var field = ((ComboBoxItem)comboBox.SelectedItem).Tag as string;
            if (grid != null && grid.ExpressionEditors.Contains(field))
            {
                grid.ExpressionEditors[field].Expression = editor.Expression;
            }
        }

        // Change ExpressionEditor settings to allow edit expression for selected DataGrid column.
        private void combobox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var field = (e.AddedItems[0] as ComboBoxItem).Tag as string;
            if (grid != null && grid.ExpressionEditors.Contains(field))
            {
                editor.DataSource = grid.ExpressionEditors[field].DataSource;
                editor.Expression = grid.ExpressionEditors[field].Expression;
            }
        }
    }

    public class DataGridEE : DataGrid, ISupportExpressions
    {
        #region Ctor
        public DataGridEE() : base()
        {
            ExpressionEditors = new ExpressionEditorCollection(this);
        }
        #endregion

        #region ISupportExpressions

        public void SetCellValue(int row, string colName, object value)
        {
            var item = Items[row];
            var propertyInfo = item?.GetType()?.GetProperty(colName);
            propertyInfo?.SetValue(item, value, null);

            Items.Refresh();
        }

        public ExpressionEditorCollection ExpressionEditors { get; }
        #endregion

        protected override void OnAutoGeneratingColumn(DataGridAutoGeneratingColumnEventArgs e)
        {
            if ( e.PropertyType == typeof(double))
            {
                ((DataGridTextColumn)e.Column).Binding.StringFormat = "F2";
            }
            base.OnAutoGeneratingColumn(e);
        }
    }
}
