﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using C1.Chart;
using C1.Chart.Finance;
using TechnicalIndicators.Object;
using System.Windows.Media;

namespace TechnicalIndicators.Partial.CustomControls.CustomIndicator
{
    public class ADX : IndicatorCharts.IndicatorChartBase
    {

        static ADX()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ADX), new FrameworkPropertyMetadata(typeof(ADX)));
        }

        internal DIPSeries DIP
        {
            get;
            private set;
        }

        internal DINSeries DIN
        {
            get;
            private set;
        }

        internal ADXSeries ADXSeries
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            if (this.IndicatorChart != null)
            {
                this.IndicatorChart.Rendered += (o, e) =>
                {
                    RangeSelectorChanged(ViewModel.ViewModel.Instance.LowerValue.Value, ViewModel.ViewModel.Instance.UpperValue.Value);
                };
            }

            C1.WPF.Chart.Axis ax = new C1.WPF.Chart.Axis();
            DIP = this.Template.FindName("dip", this) as DIPSeries;
            DIN = this.Template.FindName("din", this) as DINSeries;
            ADXSeries = this.Template.FindName("adx", this) as ADXSeries;


            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;
            if(quote != null)
                ADXCalculator.Instance.Source = quote.Data;
            if (DIP != null)
            { 
                DIP.Calculator = ADXCalculator.Instance;
                DIP.AxisY = ax;
            }
            if (DIN != null)
            {
                DIN.Calculator = ADXCalculator.Instance;
                DIN.AxisY = ax;
            }
            if (ADXSeries != null)
            {
                ADXSeries.Calculator = ADXCalculator.Instance;
                ADXSeries.AxisY = ax;
            }

            this.IndicatorChart.ItemsSource = this.Source;

            ViewModel.ViewModel.Instance.PropertyChanged += (o, e) =>
            {
                if (e.PropertyName == "CurrectQuote")
                {
                    quote = ViewModel.ViewModel.Instance.CurrectQuote;
                    ADXCalculator.Instance.Source = quote.Data;
                    this.IndicatorChart.ItemsSource = this.Source;
                    
                    RangeSelectorChanged(ViewModel.ViewModel.Instance.LowerValue.Value, ViewModel.ViewModel.Instance.UpperValue.Value);
                }
            };

            

            BindingSettingsParams(DIP, typeof(DIPSeries), "ADX",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Period", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush), "D+"),
                }
            );
            BindingSettingsParams(DIN, typeof(DINSeries), "ADX",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Period", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush), "D-"),
                }
            );
            BindingSettingsParams(ADXSeries, typeof(ADXSeries), "ADX",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Period", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush), "ADX"),
                }
            );
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (DIP != null)
            {
                IEnumerable<double> quoteData = DIP.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high, DIP.Period));
            }
            if (DIN != null)
            {
                IEnumerable<double> quoteData = DIN.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high, DIN.Period));
            }
            if (ADXSeries != null)
            {
                IEnumerable<double> quoteData = ADXSeries.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high, ADXSeries.Period * 2 - 1));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e, int period)
        {
            QuoteRange qr = null;

            var start = Convert.ToInt32(s) - period;
            var len = Convert.ToInt32(e) - Convert.ToInt32(s);
            IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
            if (quoteCache.Any())
            {
                qr = new QuoteRange();
                var min = quoteCache.Min();
                var max = quoteCache.Max();
                var sub = (max - min) * .25;
                qr.Min = min - sub;
                qr.Max = max + sub;
            }
            return qr;
        }

        
        public IEnumerable<ADXDataPoint> Source
        {
            get
            {
                var vals = ADXCalculator.Instance.DataPoints;
                return vals;
            }
        }
    }
    

}
