﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.CustomIndicator
{
    public class MassIndex : IndicatorCharts.IndicatorChartBase
    {
        static MassIndex()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(MassIndex), new FrameworkPropertyMetadata(typeof(MassIndex)));
        }

        internal MassIndexSeries Series
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as MassIndexSeries;

            this.IndicatorChart.Rendered += (o, e) =>
            {
                RangeSelectorChanged(ViewModel.ViewModel.Instance.LowerValue.Value, ViewModel.ViewModel.Instance.UpperValue.Value);
            };

            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;
            if (quote != null)
            {
                IndicatorChart.ItemsSource = quote.Data;
            }
            ViewModel.ViewModel.Instance.PropertyChanged += (o, e) =>
            {
                if (e.PropertyName == "CurrectQuote")
                {
                    quote = ViewModel.ViewModel.Instance.CurrectQuote;
                    if (quote != null)
                    {
                        IndicatorChart.ItemsSource = quote.Data;
                    }
                }
            };
            
            BindingSettingsParams(Series, typeof(MassIndexSeries), "MassIndex",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Period", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush)),
                }
            );
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> quoteData = Series.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;
            
            var offset = Series.Period + Series.Period - 2 + 25 - 1;
            var start = Convert.ToInt32(s) < offset ? 0 : (Convert.ToInt32(s)- offset);
            var len = Convert.ToInt32(e) - Convert.ToInt32(s);
            IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
            if (quoteCache.Any())
            {
                qr = new QuoteRange();
                var min = quoteCache.Min();
                var max = quoteCache.Max();
                qr.Min = min;
                qr.Max = max;
            }
            return qr;
        }
    }
}
