﻿using C1.Chart;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;

namespace TechnicalIndicators.Partial.CustomControls.CustomIndicator
{
    public class MassIndexSeries : C1.WPF.Chart.Finance.FinancialSeries
    {
        public MassIndexSeries()
        {
            this.BindingX = "XVals";
            this.Binding = "YVals";
        }


        public int Period
        {
            get { return (int)GetValue(PeriodProperty); }
            set { SetValue(PeriodProperty, value); }
        }

        public static readonly DependencyProperty PeriodProperty =
            DependencyProperty.Register("Period", typeof(int), typeof(MassIndexSeries),
            new PropertyMetadata(14, OnPeriodChanged));


        #region Implementation
        private static void OnPeriodChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MassIndexSeries mi = d as MassIndexSeries;
            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;

            var source = mi.Create(quote.Data);

            mi.ItemsSource = source;
        }

        public override double[] GetValues(int dim)
        {
            double[] result = null;
            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;
            if (quote != null)
            {
                var source = Create(quote.Data);
                

                switch (dim)
                {
                    case 0:
                        result = source.Select(p => p.YVals).ToArray();
                        break;
                    case 1:
                        result = source.Select(p => p.XVals).ToArray();
                        break;
                    default:
                        break;
                }
            }
            return result;
        }

        #endregion

        internal IEnumerable<DataPoint> Create(IEnumerable<Object.QuoteData> quoteData)
        {
            var dps = CalculateExpotential(quoteData.ToArray());
            return dps;
        }

        List<DataPoint> CalculateExpotential(Object.QuoteData[] qdata)
        {
            var dps = new List<DataPoint>();

            var len = qdata.Length;
            
            var massIndex = new double[len];

            double ema = 0.0, ema2 = 0.0;
            for (int i = 0; i < this.Period; i++)
            {
                massIndex[i] = double.NaN;
                ema += qdata[i].High - qdata[i].Low;
            }

            ema /= this.Period;
            ema2 = ema;

            for (int i = this.Period; i < this.Period + this.Period - 1; i++)
            {
                massIndex[i] = double.NaN;
                ema = ema + 2.0 / (this.Period + 1) * (qdata[i].High - qdata[i].Low - ema);
                ema2 += ema;
            }

            ema2 /= this.Period;

            double emaRatio = 0.0;
            double[] emaRatios = new double[25];      // 25 element circular buffer

            for (int i = this.Period + this.Period - 1; i < this.Period + this.Period - 1 + 25; i++)
            {
                massIndex[i] = double.NaN;
                ema = ema + 2.0 / (this.Period + 1) * (qdata[i].High - qdata[i].Low - ema);
                ema2 = ema2 + 2.0 / (this.Period + 1) * (ema - ema2);
                emaRatio += ema / ema2;
                emaRatios[i % 25] = ema / ema2;
            }
            massIndex[this.Period + this.Period - 2 + 25] = emaRatio;

            dps.Add(new DataPoint()
            {
                XVals = (this.Period + this.Period - 2 + 25),
                YVals = emaRatio,
            });
            for (int i = this.Period + this.Period - 1 + 25; i < len; i++)
            {
                ema = ema + 2.0 / (this.Period + 1) * (qdata[i].High - qdata[i].Low - ema);
                ema2 = ema2 + 2.0 / (this.Period + 1) * (ema - ema2);
                emaRatio = ema / ema2;
                massIndex[i] = massIndex[i - 1] - emaRatios[i % 25] + emaRatio;
                emaRatios[i % 25] = emaRatio;

                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = massIndex[i],
                });
            }

            return dps;

        }

    }
    
    public class DataPoint
    {
        public double XVals { get; set; }
        public double YVals { get; set; }
        public double ZVals { get; set; }
    }
}
