﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.CustomIndicator
{
    public class PivotPoint : IndicatorCharts.IndicatorChartBase
    {
        static PivotPoint()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(PivotPoint), new FrameworkPropertyMetadata(typeof(PivotPoint)));
        }

        internal PivotPointSeries Series
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries R1
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries R2
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries R3
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries S1
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries S2
        {
            get;
            private set;
        }
        internal C1.WPF.Chart.Finance.FinancialSeries S3
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as PivotPointSeries;

            R1 = this.Template.FindName("r1", this) as PivotPointSeries;
            R2 = this.Template.FindName("r2", this) as PivotPointSeries;
            R3 = this.Template.FindName("r3", this) as PivotPointSeries;
            S1 = this.Template.FindName("s1", this) as PivotPointSeries;
            S2 = this.Template.FindName("s2", this) as PivotPointSeries;
            S3 = this.Template.FindName("s3", this) as PivotPointSeries;


            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;
            if (quote != null)
            {
                IndicatorChart.ItemsSource = quote.Data;
            }
            ViewModel.ViewModel.Instance.PropertyChanged += (o, e) =>
            {
                if (e.PropertyName == "CurrectQuote")
                {
                    quote = ViewModel.ViewModel.Instance.CurrectQuote;
                    if (quote != null)
                    {
                        IndicatorChart.ItemsSource = quote.Data;
                    }
                }
            };

            BindingSettingsParams(Series, typeof(PivotPointSeries), "PivotPoint",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Style.Stroke", typeof(Brush)),
                }
            );
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> quoteData = Series.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }

            if (R1 != null)
            {
                IEnumerable<double> quoteData = R1.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            if (R2 != null)
            {
                IEnumerable<double> quoteData = R2.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            if (R3 != null)
            {
                IEnumerable<double> quoteData = R3.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            if (S1 != null)
            {
                IEnumerable<double> quoteData = S1.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            if (S2 != null)
            {
                IEnumerable<double> quoteData = S2.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            if (S3 != null)
            {
                IEnumerable<double> quoteData = S3.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;

            var start = Convert.ToInt32(s) - 1;
            var len = Convert.ToInt32(e) - Convert.ToInt32(s);
            IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
            if (quoteCache.Any())
            {
                qr = new QuoteRange();
                var min = quoteCache.Min();
                var max = quoteCache.Max();
                qr.Min = min;
                qr.Max = max;
            }
            return qr;
        }
    }
}
