﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.CustomIndicator
{
    public abstract class PivotPointSeriesBase : C1.WPF.Chart.Finance.FinancialSeries
    {
        public override double[] GetValues(int dim)
        {
            double[] result = null;
            Object.Quote quote = ViewModel.ViewModel.Instance.CurrectQuote;
            if (quote != null)
            {
                var source = Create(quote.Data);

                Queue<double> ds = new Queue<double>();
                for (int i = 0; i < source.Count(); i++)
                {
                    ds.Enqueue(i);
                }

                switch (dim)
                {
                    case 0:
                        result = source.Select(p => p.YVals).ToArray();
                        break;
                    case 1:
                        result = ds.ToArray();
                        break;
                    default:
                        break;
                }
            }
            return result;
        }
        protected abstract IEnumerable<DataPoint> Create(IEnumerable<Object.QuoteData> quoteData);

    }

    public class PivotPointSeries : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<Object.QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();

            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = pivot,
                });
                i++;
            }

            return dps;
        }

    }
    public class R1Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = 2 * pivot - data.Low,
                });
                i++;
            }
            return dps;
        }
    }
    public class R2Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = pivot + data.High - data.Low,
                });
                i++;
            }
            return dps;
        }
    }
    public class R3Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals =  data.High + 2 * (pivot - data.Low),
                });
                i++;
            }
            return dps;
        }
    }
    public class S1Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = 2 * pivot - data.High,
                });
                i++;
            }
            return dps;
        }
    }
    public class S2Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = pivot - data.High + data.Low,
                });
                i++;
            }
            return dps;
        }
    }
    public class S3Series : PivotPointSeriesBase
    {
        protected override IEnumerable<DataPoint> Create(IEnumerable<QuoteData> quoteData)
        {
            var dps = new List<DataPoint>();
            int i = 0;
            foreach (var data in quoteData)
            {
                var pivot = (data.High + data.Low + data.Close) / 3;
                dps.Add(new DataPoint()
                {
                    XVals = i,
                    YVals = data.Low - 2 * (data.High - pivot),
                });
                i++;
            }
            return dps;
        }
    }
}
