﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.IndicatorCharts
{
    public abstract class IndicatorChartBase : Control, IDisposable
    {
        private C1.WPF.Chart.Axis xAxis;
        protected C1.WPF.Chart.Finance.C1FinancialChart IndicatorChart
        {
            get;
            private set;
        }

        public IndicatorChartBase() 
        {
            ViewModel.ViewModel.Instance.PropertyChanged += Instance_PropertyChanged;
        }

        private void Instance_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "CurrectQuote")
            {
                if (ViewModel.ViewModel.Instance.LowerValue != null && ViewModel.ViewModel.Instance.UpperValue != null)
                    RangeSelectorChanged(ViewModel.ViewModel.Instance.LowerValue.Value, ViewModel.ViewModel.Instance.UpperValue.Value);
            }
            else if (e.PropertyName == "LowerValue")
            {
                this.Dispatcher.BeginInvoke(new Action(() =>
                {
                    LowerValue = ViewModel.ViewModel.Instance.LowerValue.Value;
                }));
            }
            else if (e.PropertyName == "UpperValue")
            {
                this.Dispatcher.BeginInvoke(new Action(() =>
                {
                    UpperValue = ViewModel.ViewModel.Instance.UpperValue.Value;
                }));
            }
        }

        public void Dispose()
        {
            ViewModel.ViewModel.Instance.PropertyChanged -= Instance_PropertyChanged;
        }


        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            xAxis = this.Template.FindName("xAxis", this) as C1.WPF.Chart.Axis;
            IndicatorChart = this.Template.FindName("indicatorChart", this) as C1.WPF.Chart.Finance.C1FinancialChart;
            if (xAxis != null)
            {
                xAxis.AxisLine = xAxis.Labels = this.IsShowAxis;
                xAxis.Min = LowerValue;
                xAxis.Max = UpperValue;
            }
        }

        //Binding view mode property setting panel
        protected void BindingSettingsParams(object source, Type srcType, string srckey, IEnumerable<Data.PropertyParam> properties)
        {
            Utilities.Helper.BindingSettingsParams(IndicatorChart, source, srcType, srckey, properties);
        }

        public IEnumerable<Object.QuoteData> ItemSource
        {
            get { return (IEnumerable<Object.QuoteData>)this.GetValue(ItemSourceProperty); }
            set { this.SetValue(ItemSourceProperty, value); }
        }

        public static DependencyProperty ItemSourceProperty = DependencyProperty.Register(
            "ItemSource",
            typeof(IEnumerable<Object.QuoteData>),
            typeof(IndicatorChartBase),
            new PropertyMetadata(null));


        public Thickness PlotMargin
        {
            get { return (Thickness)this.GetValue(PlotMarginProperty); }
            set { this.SetValue(PlotMarginProperty, value); }
        }

        public static DependencyProperty PlotMarginProperty = DependencyProperty.Register(
            "PlotMargin",
            typeof(Thickness),
            typeof(IndicatorChartBase),
            new PropertyMetadata(new Thickness(0,0,50,0)));
        
        public bool IsShowAxis
        {
            get { return (bool)this.GetValue(IsShowAxisProperty); }
            set { this.SetValue(IsShowAxisProperty, value); }
        }
        public static DependencyProperty IsShowAxisProperty = DependencyProperty.Register(
            "IsShowAxis",
            typeof(bool),
            typeof(IndicatorChartBase),
            new PropertyMetadata(false, new PropertyChangedCallback(
                (o, e) =>
                {
                    var chart = o as IndicatorChartBase;
                    if (chart != null && chart.xAxis != null)
                    {
                        chart.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            chart.xAxis.AxisLine = chart.xAxis.Labels = Convert.ToBoolean(e.NewValue);
                        }));
                    }
                }
                )));

        public double LowerValue
        {
            get { return (double)this.GetValue(LowerValueProperty); }
            set { this.SetValue(LowerValueProperty, value); }
        }

        public static DependencyProperty LowerValueProperty = DependencyProperty.Register(
            "LowerValue",
            typeof(double),
            typeof(IndicatorChartBase),
            new PropertyMetadata(ViewModel.ViewModel.Instance.LowerValue == null ? 0 : ViewModel.ViewModel.Instance.LowerValue.Value, new PropertyChangedCallback(
                (o, e) =>
                {
                    var chart = o as IndicatorChartBase;
                    if (chart.xAxis != null)
                    {
                        chart.xAxis.Min = chart.LowerValue;
                        chart.RangeSelectorChanged(chart.LowerValue, chart.UpperValue);
                    }
                })));

        public double UpperValue
        {
            get { return (double)this.GetValue(UpperValueProperty); }
            set { this.SetValue(UpperValueProperty, value); }
        }

        public static DependencyProperty UpperValueProperty = DependencyProperty.Register(
            "UpperValue",
            typeof(double),
            typeof(IndicatorChartBase),
            new PropertyMetadata(ViewModel.ViewModel.Instance.UpperValue == null ? 0 : ViewModel.ViewModel.Instance.UpperValue.Value, new PropertyChangedCallback(
                (o, e) =>
                {
                    var chart = o as IndicatorChartBase;
                    if (chart.xAxis != null)
                    {
                        chart.xAxis.Max = chart.UpperValue;
                        chart.RangeSelectorChanged(chart.LowerValue, chart.UpperValue);
                    }

                })));

        
        public C1.WPF.Chart.RenderMode RenderMode
        {
            get { return (C1.WPF.Chart.RenderMode)GetValue(RenderModeProperty); }
            set { SetValue(RenderModeProperty, value); }
        }

        /// <summary>
        /// Identifies the RenderMode dependency property.
        /// </summary>
        public static readonly DependencyProperty RenderModeProperty =
            DependencyProperty.Register("RenderMode", typeof(C1.WPF.Chart.RenderMode), typeof(IndicatorChartBase),
            new PropertyMetadata(ViewModel.ViewModel.Instance.RenderMode));

        protected void RangeSelectorChanged(double low, double high)
        {
            if (IndicatorChart != null)
            {
                IndicatorChart.BeginUpdate();

                IEnumerable<QuoteRange> ranges = GetYRange(low, high);
                if (ranges != null && ranges.Any())
                {
                    if (IndicatorChart != null)
                    {
                        this.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            IndicatorChart.AxisY.Min = ranges.Min(p => { return p == null ? int.MaxValue : p.Min; });
                            IndicatorChart.AxisY.Max = ranges.Max(p => { return p == null ? int.MinValue : p.Max; });
                        }));
                    }
                }
                IndicatorChart.EndUpdate();
            }
        }
        public abstract IEnumerable<QuoteRange> GetYRange(double low, double high);

    }
}
