﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.IndicatorCharts
{
    public class MacdChart : IndicatorChartBase
    {
        static MacdChart()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(MacdChart), new FrameworkPropertyMetadata(typeof(MacdChart)));
        }


        internal C1.WPF.Chart.Finance.Macd Series
        {
            get;
            private set;
        }

        internal C1.WPF.Chart.Finance.MacdHistogram Histogram
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as C1.WPF.Chart.Finance.Macd;
            Histogram = this.Template.FindName("histogram", this) as C1.WPF.Chart.Finance.MacdHistogram;

            if (Histogram != null && Histogram.Style != null)
            {
                Binding binding = new Binding("Stroke");
                binding.Source = Histogram.Style;
                binding.Converter = new Converter.BrushTransparencyConverter();
                binding.ConverterParameter = .5;
                BindingOperations.SetBinding(Histogram.Style, C1.WPF.Chart.ChartStyle.FillProperty, binding);
            }

            BindingSettingsParams(Series, typeof(C1.WPF.Chart.Finance.Macd), "Macd",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("FastPeriod", typeof(int)),
                    new Data.PropertyParam("SlowPeriod", typeof(int)),
                    new Data.PropertyParam("SmoothingPeriod", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush), "Series"),
                }
            );

            BindingSettingsParams(Histogram, typeof(C1.WPF.Chart.Finance.MacdHistogram), "Macd",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Style.Stroke", typeof(Brush), "Histogram"),
                }
            );
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> mData = Series.MacdVals;
                IEnumerable<double> sData = Series.SignalVals;
                if (mData != null && sData != null)
                {
                    ranges.Enqueue(GetSymbolYRange(mData, low, high));
                    ranges.Enqueue(GetSymbolYRange(sData, low, high));
                }
                else
                {
                    ranges.Enqueue(GetSymbolYRange(Series.GetValues(0), low, high));
                }
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;
            if (quoteData.Any())
            {
                var len = Convert.ToInt32(e) - Convert.ToInt32(s);
                var start = quoteData.Count() - len;
                IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
                if (quoteCache.Any())
                {
                    qr = new QuoteRange();
                    var min = quoteCache.Min();
                    var max = quoteCache.Max();
                    var sub = (max - min) * .25;
                    qr.Min = min - sub;
                    qr.Max = max + sub;
                }
            }
            return qr;
        }

    }
}
