﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Data;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.IndicatorCharts
{
    public abstract class SingleSeriesIndicatorChart<T>: IndicatorChartBase
        where T : C1.WPF.Chart.Finance.IndicatorBase
    {
        internal T Series
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as T;
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> quoteData = Series.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;

            var start = Convert.ToInt32(s) - Series.Period;
            var len = Convert.ToInt32(e) - Convert.ToInt32(s);
            IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
            if (quoteCache.Any())
            {
                qr = new QuoteRange();
                var min = quoteCache.Min();
                var max = quoteCache.Max();
                var sub = (max - min) * .25;
                qr.Min = min - sub;
                qr.Max = max + sub;
            }
            return qr;
        }


        

    }
}
