﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.IndicatorCharts
{
    public class StochasticChart : IndicatorChartBase
    {
        static StochasticChart()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(StochasticChart), new FrameworkPropertyMetadata(typeof(StochasticChart)));
        }

        internal C1.WPF.Chart.Finance.Stochastic Series
        {
            get;
            private set;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as C1.WPF.Chart.Finance.Stochastic;
            
            BindingSettingsParams(Series, typeof(C1.WPF.Chart.Finance.Stochastic), "Stochastic",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("KPeriod", typeof(int)),
                    new Data.PropertyParam("DPeriod", typeof(int)),
                    new Data.PropertyParam("SmoothingPeriod", typeof(int)),
                    new Data.PropertyParam("Style.Stroke", typeof(Brush)),
                }
            );
        }

        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> kData = Series.KValues;
                if (kData == null)
                    return null;
                    ranges.Enqueue(GetSymbolYRange(kData, low, high));
                
                IEnumerable<double> dData = Series.DValues;
                if (dData == null)
                    return null;
                ranges.Enqueue(GetSymbolYRange(dData, low, high));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;
            if (quoteData.Any())
            {
                var len = Convert.ToInt32(e) - Convert.ToInt32(s);
                var start = quoteData.Count() - len;
                IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
                if (quoteCache.Any())
                {
                    qr = new QuoteRange();
                    var min = quoteCache.Min();
                    var max = quoteCache.Max();
                    var sub = (max - min) * .25;
                    qr.Min = min - sub;
                    qr.Max = max + sub;
                }
            }
            return qr;
        }
    }
}
