﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TechnicalIndicators.Object;

namespace TechnicalIndicators.Partial.CustomControls.IndicatorCharts
{
    public class VolumeChart : IndicatorChartBase
    {
        static VolumeChart()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(VolumeChart), new FrameworkPropertyMetadata(typeof(VolumeChart)));
        }

        internal C1.WPF.Chart.Series Series
        {
            get;
            private set;
        }

        
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Series = this.Template.FindName("series", this) as C1.WPF.Chart.Series;

            if (Series != null && Series.Style != null)
            {
                Binding binding = new Binding("Stroke");
                binding.Source = Series.Style;
                binding.Converter = new Converter.BrushTransparencyConverter();
                binding.ConverterParameter = .5;
                BindingOperations.SetBinding(Series.Style, C1.WPF.Chart.ChartStyle.FillProperty, binding);
            }

            BindingSettingsParams(Series, typeof(C1.WPF.Chart.Series), "Volume",
                new Data.PropertyParam[]
                {
                    new Data.PropertyParam("Style.Stroke", typeof(Brush)),
                }
            );
        }
        public override IEnumerable<QuoteRange> GetYRange(double low, double high)
        {
            Queue<QuoteRange> ranges = new Queue<QuoteRange>();
            if (Series != null)
            {
                IEnumerable<double> quoteData = Series.GetValues(0);
                ranges.Enqueue(GetSymbolYRange(quoteData, low, high));
            }
            return ranges;
        }

        public QuoteRange GetSymbolYRange(IEnumerable<double> quoteData, double s, double e)
        {
            QuoteRange qr = null;

            var start = Convert.ToInt32(s);
            var len = Convert.ToInt32(e) - Convert.ToInt32(s);
            IEnumerable<double> quoteCache = quoteData.Skip(start).Take(len);
            if (quoteCache.Any())
            {
                qr = new QuoteRange();
                var min = quoteCache.Min();
                var max = quoteCache.Max();
                qr.Min = min;
                qr.Max = max;
            }
            return qr;
        }

    }
}
