﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TechnicalIndicators.Partial.UserControls
{
    /// <summary>
    /// Interaction logic for IndicatorViews.xaml
    /// </summary>
    public partial class IndicatorViews : UserControl
    {
        public IndicatorViews()
        {
            InitializeComponent();

            ViewModel.ViewModel.Instance.IndicatorCharts.CollectionChanged += (o, e) =>
            {
                if (e.OldItems!=null && e.OldItems.Count > 0)
                {
                    foreach (Type type in e.OldItems)
                    {
                        foreach (CustomControls.IndicatorCharts.IndicatorChartBase item in container.Children)
                        {
                            if (item.GetType() == type)
                            {
                                container.Children.Remove(item);
                                item.Dispose();
                                break;
                            }
                        }
                    }
                }
                else if (e.NewItems != null && e.NewItems.Count > 0)
                {
                    foreach (Type type in e.NewItems)
                    {
                        CustomControls.IndicatorCharts.IndicatorChartBase chart = type.Assembly.CreateInstance(type.FullName) as CustomControls.IndicatorCharts.IndicatorChartBase;
                        
                        chart.Name = type.Name;
                        chart.Height = 50;
                        
                        Binding srcBinding = new Binding();
                        srcBinding.Path = new PropertyPath("CurrectQuote.Data");
                        srcBinding.Source = ViewModel.ViewModel.Instance;
                        BindingOperations.SetBinding(chart, CustomControls.IndicatorCharts.IndicatorChartBase.ItemSourceProperty, srcBinding);

                        Binding lowerBinding = new Binding();
                        lowerBinding.Path = new PropertyPath("LowerValue");
                        lowerBinding.Source = ViewModel.ViewModel.Instance;
                        BindingOperations.SetBinding(chart, CustomControls.IndicatorCharts.IndicatorChartBase.LowerValueProperty, lowerBinding);
                        Binding upperBinding = new Binding();
                        upperBinding.Path = new PropertyPath("UpperValue");
                        upperBinding.Source = ViewModel.ViewModel.Instance;
                        BindingOperations.SetBinding(chart, CustomControls.IndicatorCharts.IndicatorChartBase.UpperValueProperty, upperBinding);
                        
                        container.Children.Add(chart);
                    }
                }

                foreach (CustomControls.IndicatorCharts.IndicatorChartBase chart in container.Children)
                {
                    chart.BeginInit();
                    if (chart == container.Children[container.Children.Count - 1])
                    {
                        //last one
                        chart.Height = 50 + ChartWithAxisPlotMargin.Bottom - ChartPlotMargin.Bottom;
                        chart.PlotMargin = ChartWithAxisPlotMargin;
                        chart.IsShowAxis = true;
                    }
                    else
                    {
                        chart.Height = 50;
                        chart.PlotMargin = ChartPlotMargin;
                        chart.IsShowAxis = false;
                    }
                    chart.EndInit();
                }

            };
        }

        private Thickness? chartPlotMargin = null;
        protected Thickness ChartPlotMargin
        {
            get
            {
                if (chartPlotMargin == null)
                {
                    chartPlotMargin = (Thickness)this.FindResource("ChartPlotMargin");
                }
                return chartPlotMargin.Value;
            }
        }

        private Thickness? chartWithAxisPlotMargin = null;
        protected Thickness ChartWithAxisPlotMargin
        {
            get
            {
                if (chartWithAxisPlotMargin == null)
                {
                    chartWithAxisPlotMargin = (Thickness)this.FindResource("ChartWithAxisPlotMargin");
                }
                return chartWithAxisPlotMargin.Value;
            }
        }




    }
}
