﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace TechnicalIndicators.ViewModel
{
    public struct IndicatorPalettes
    {
        public static Color Red = Color.FromArgb(255, 244,67,54);
        public static Color Pink = Color.FromArgb(255, 233, 30, 99);

        public static Color Purple = Color.FromArgb(255, 156, 39, 176);
        public static Color DeepPurple = Color.FromArgb(255, 103, 58, 183);
        public static Color Indigo = Color.FromArgb(255, 64, 82, 180);
        public static Color Blue = Color.FromArgb(255, 34, 150, 243);
        public static Color LightBlue = Color.FromArgb(255, 4, 168, 244);
        public static Color Cyan = Color.FromArgb(255, 2, 188, 212);
        public static Color Teal = Color.FromArgb(255, 0, 149, 136);
        public static Color Green = Color.FromArgb(255, 76, 175, 80);
        public static Color LightGreen = Color.FromArgb(255, 138, 195, 75);
        public static Color Lime = Color.FromArgb(255, 205, 220, 57);

        public static Color Yellow = Color.FromArgb(255, 255, 235, 60);
        public static Color Amber = Color.FromArgb(255, 255, 193, 7);
        public static Color Orange = Color.FromArgb(255, 255, 155, 3);
        public static Color DeepOrange = Color.FromArgb(255, 255, 87, 34);
        public static Color Brown = Color.FromArgb(255, 121, 85, 72);
        public static Color Grey = Color.FromArgb(255, 158, 158, 158);
        public static Color BlueGrey = Color.FromArgb(255, 96, 125, 139);

        ///00B050   ff1d25
        public static Color StockGreen = Color.FromArgb(255, 00, 176, 80);
        public static Color StockRed = Color.FromArgb(255, 255, 29, 37);
    }

    public class ViewModel: INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private ViewModel()
        {
            Data.DataService.GetQuotesAsync(Symbols,
                (o, e) =>
                {
                    Quotes = e.Quotes;
                }
                );
        }
        
        private static ViewModel instance;
        public static ViewModel Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new ViewModel();
                }
                return instance;
            }
        }

        private C1.WPF.Chart.RenderMode renderMode = C1.WPF.Chart.RenderMode.Direct2D;
        public C1.WPF.Chart.RenderMode RenderMode
        {
            get
            {
                return renderMode;
            }
        }

        private Dictionary<string, string> symbols;
        public Dictionary<string, string> Symbols
        {
            get
            {
                if (symbols == null)
                {
                    symbols = new Dictionary<string, string>();
                    symbols.Add("AAPL", "Apple");
                    symbols.Add("MSFT", "Microsoft");
                    symbols.Add("INTC", "Intel");
                    symbols.Add("CSCO", "Cisco");
                    symbols.Add("GOOG", "Google");
                    symbols.Add("AMZN", "Amazon");
                    symbols.Add("EBAY", "eBay");
                    symbols.Add("DVMT", "Dell");
                    symbols.Add("ORCL", "Oracle");
                    symbols.Add("VOD", "Vodafone");
                }
                return symbols;
            }
        }


        private IEnumerable<Object.Quote> quotes;
        public IEnumerable<Object.Quote> Quotes
        {
            get
            {
                return quotes;
            }
            private set
            {
                quotes = value;
                OnPropertyChanged("Quotes");
            }
        }

        private bool isLoaded = false;
        public bool IsLoaded
        {
            get
            {
                return isLoaded;
            }
            internal set
            {
                isLoaded = value;
                OnPropertyChanged("IsLoaded");
            }
        }

        private Object.Quote _currectQuote;
        public Object.Quote CurrectQuote
        {
            get
            {
                return _currectQuote;
            }
            set
            {
                _currectQuote = value;
                OnPropertyChanged("CurrectQuote");
            }

        }
        
        private double? _lowValue = null;
        public double? LowerValue
        {
            get
            {
                return _lowValue;
            }
            set
            {
                _lowValue = value;
                OnPropertyChanged("LowerValue");
            }
        }

        private double? _upperValue = null;
        public double? UpperValue
        {
            get
            {
                return _upperValue;
            }
            set
            {
                _upperValue = value;
                OnPropertyChanged("UpperValue");
            }
        }

        private IEnumerable<C1.Chart.Finance.FinancialChartType> category_01 = new C1.Chart.Finance.FinancialChartType[]
        {
            C1.Chart.Finance.FinancialChartType.Candlestick,
            C1.Chart.Finance.FinancialChartType.ArmsCandleVolume,
            C1.Chart.Finance.FinancialChartType.CandleVolume,
            C1.Chart.Finance.FinancialChartType.HeikinAshi,
            C1.Chart.Finance.FinancialChartType.HighLowOpenClose,
            C1.Chart.Finance.FinancialChartType.Line,
            C1.Chart.Finance.FinancialChartType.LineSymbols,
        };
        private IEnumerable<C1.Chart.Finance.FinancialChartType> category_02 = new C1.Chart.Finance.FinancialChartType[]
        {
            C1.Chart.Finance.FinancialChartType.Kagi,
            C1.Chart.Finance.FinancialChartType.Renko,
        };

        private C1.Chart.Finance.FinancialChartType chartType = C1.Chart.Finance.FinancialChartType.Candlestick;
        public C1.Chart.Finance.FinancialChartType ChartType
        {
            get { return this.chartType; }
            set
            {
                if (this.chartType != value)
                {
                    if (category_01.Contains(this.chartType) ^ category_01.Contains(value))
                    {
                        OnPropertyChanged("ChartTypeCategory");
                    }
                    this.chartType = value;
                    OnPropertyChanged("ChartType");
                }
            }
        }

        private System.Collections.ObjectModel.ObservableCollection<Type> indicatorCharts;
        public System.Collections.ObjectModel.ObservableCollection<Type> IndicatorCharts
        {
            get
            {
                if (indicatorCharts == null)
                {
                    indicatorCharts = new System.Collections.ObjectModel.ObservableCollection<Type>();
                    indicatorCharts.CollectionChanged += (o, e)=>
                    {
                        OnPropertyChanged("IndicatorCharts");
                    };
                }
                return this.indicatorCharts;
            }
        }

        private Dictionary<string, IEnumerable<Data.SettingParam>> settings;
        public Dictionary<string, IEnumerable<Data.SettingParam>> Settings
        {
            get
            {
                if (settings == null)
                {
                    settings = new Dictionary<string, IEnumerable<Data.SettingParam>>();
                    InitSettings(settings);
                }
                return settings;
            }
        }


        readonly static uint _defalutPeriod = 14;
        private void InitSettings(Dictionary<string, IEnumerable<Data.SettingParam>> settings)
        {
            settings
                .Add("ATR",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("Period", typeof(uint), _defalutPeriod),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
                }
                );
            settings
                .Add("RSI",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("Period", typeof(uint), _defalutPeriod),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.DeepPurple), "Color"),
                }
                );
            settings
                .Add("CCI",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("Period", typeof(uint), _defalutPeriod),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Brown), "Color"),
                }
                );
            settings
                .Add("WilliamsR",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("Period", typeof(uint), _defalutPeriod),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Green), "Color"),
                }
                );
            settings
                .Add("Macd",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("FastPeriod", typeof(uint), 12, "Fast period"),
                    new Data.SettingParam("SlowPeriod", typeof(uint), 26, "Slow period"),
                    new Data.SettingParam("SmoothingPeriod", typeof(uint), 9, "Signal smoothing period"),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.StockRed), "Series", "Series Color"),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Green), "Histogram", "Histogram Color"),
                }
                );
            settings
                .Add("Stochastic",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("KPeriod", typeof(uint), 14, "K period"),
                    new Data.SettingParam("DPeriod", typeof(uint), 3, "D period"),
                    new Data.SettingParam("SmoothingPeriod", typeof(uint), 1, "Signal smoothing period"),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.LightBlue), "Color"),
                }
                );
            settings
                .Add("Volume",
                new Data.SettingParam[]
                {
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Blue), "Color"),
                }
                );


            settings
               .Add("Kagi",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("RangeMode", typeof(C1.Chart.Finance.RangeMode), C1.Chart.Finance.RangeMode.Fixed, "Range Mode", 
                        new object[]{ C1.Chart.Finance.RangeMode.ATR, C1.Chart.Finance.RangeMode.Fixed, C1.Chart.Finance.RangeMode.Percentage }
                    ),
                    new Data.SettingParam("ReversalAmount", typeof(uint), 2, "Reversal Amount"),
                    new Data.SettingParam("DataFields", typeof(C1.Chart.Finance.DataFields), C1.Chart.Finance.DataFields.Close, "Data Fields",
                        new object[]{ C1.Chart.Finance.DataFields.High, C1.Chart.Finance.DataFields.Low, C1.Chart.Finance.DataFields.Open, C1.Chart.Finance.DataFields.Close, C1.Chart.Finance.DataFields.HighLow, C1.Chart.Finance.DataFields.HL2, C1.Chart.Finance.DataFields.HLC3, C1.Chart.Finance.DataFields.HLOC4 }
                    )
               }
               );
            settings
               .Add("Renko",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("RangeMode", typeof(C1.Chart.Finance.RangeMode), C1.Chart.Finance.RangeMode.Fixed, "Range Mode", 
                        new object[]{ C1.Chart.Finance.RangeMode.ATR, C1.Chart.Finance.RangeMode.Fixed }
                    ),
                    new Data.SettingParam("ReversalAmount", typeof(uint), 2, "Reversal Amount"),
                    new Data.SettingParam("DataFields", typeof(C1.Chart.Finance.DataFields), C1.Chart.Finance.DataFields.Close, "Data Fields",
                        new object[]{ C1.Chart.Finance.DataFields.High, C1.Chart.Finance.DataFields.Low, C1.Chart.Finance.DataFields.Open, C1.Chart.Finance.DataFields.Close, C1.Chart.Finance.DataFields.HighLow, C1.Chart.Finance.DataFields.HL2, C1.Chart.Finance.DataFields.HLC3, C1.Chart.Finance.DataFields.HLOC4 }
                    )
               }
               );


            settings
               .Add("Bollinger",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Period", typeof(uint), 20),
                    new Data.SettingParam("Multiplier", typeof(uint), 2),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            settings
               .Add("Envelopes",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Period", typeof(uint), 20),
                    new Data.SettingParam("Size", typeof(double), 0.03),
                    new Data.SettingParam("Type", typeof(C1.Chart.Finance.MovingAverageType), C1.Chart.Finance.MovingAverageType.Simple, "Type",
                        new object[]{ C1.Chart.Finance.MovingAverageType.Simple, C1.Chart.Finance.MovingAverageType.Exponential}
                    ),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            settings
               .Add("Retracements",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Uptrend", typeof(bool), true),
                    new Data.SettingParam("LabelPosition", typeof(C1.Chart.LabelPosition), C1.Chart.LabelPosition.Left, "Label Position",
                        new object[]{ C1.Chart.LabelPosition.Left, C1.Chart.LabelPosition.Center, C1.Chart.LabelPosition.Right }
                    ),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            settings
               .Add("Arcs",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("StartX", typeof(uint), 0),
                    new Data.SettingParam("EndX", typeof(uint), 0),
                    new Data.SettingParam("StartY", typeof(uint), 0),
                    new Data.SettingParam("EndY", typeof(uint), 0),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            settings
               .Add("Fans",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("StartX", typeof(uint), 0),
                    new Data.SettingParam("EndX", typeof(uint), 0),
                    new Data.SettingParam("StartY", typeof(uint), 0),
                    new Data.SettingParam("EndY", typeof(uint), 0),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            settings
               .Add("TimeZones",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("StartX", typeof(uint), 0, 0),
                    new Data.SettingParam("EndX", typeof(uint), 3),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.BlueGrey), "Color"),
               }
               );
            
            settings
               .Add("ADX",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Period", typeof(uint), 14, 2),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.StockGreen), "D+", "D+ Color"),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.StockRed), "D-", "D- Color"),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(Colors.Black), "ADX", "ADX Color"),
               }
               );
            settings
               .Add("MassIndex",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Period", typeof(uint), 14, 2),
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Orange), "Color"),
               }
               );
            settings
               .Add("PivotPoint",
               new Data.SettingParam[]
               {
                    new Data.SettingParam("Style.Stroke", typeof(System.Windows.Media.Brush), new SolidColorBrush(IndicatorPalettes.Blue), "Color"),
               }
               );
        }

        private System.Collections.ObjectModel.ObservableCollection<Data.OverlayType> overlayTypes;
        public System.Collections.ObjectModel.ObservableCollection<Data.OverlayType> OverlayTypes
        {
            get
            {
                if (overlayTypes == null)
                {
                    overlayTypes = new System.Collections.ObjectModel.ObservableCollection<Data.OverlayType>();
                    overlayTypes.CollectionChanged += (o, e) =>
                    {
                        OnPropertyChanged("OverlayTypes");
                    };
                }
                return overlayTypes;
            }
        }


        private Data.NewAnnotationType newAnnotationType =  Data.NewAnnotationType.None;
        public Data.NewAnnotationType NewAnnotationType
        {
            get
            {
                return newAnnotationType;
            }
            set
            {
                if (newAnnotationType != value)
                {
                    newAnnotationType = value;
                    OnPropertyChanged("NewAnnotationType");
                    OnPropertyChanged("AnnotationStyle");
                }
            }
        }

        private Dictionary<Data.NewAnnotationType, Data.AnnotationStyle> _annotationStyles = new Dictionary<Data.NewAnnotationType, Data.AnnotationStyle>();
        
        public Data.AnnotationStyle AnnotationStyle
        {
            get
            {
                if (!_annotationStyles.Keys.Contains(this.NewAnnotationType))
                {
                    _annotationStyles[this.NewAnnotationType] = new Data.AnnotationStyle();
                    _annotationStyles[this.NewAnnotationType].PropertyChanged += (o, e) =>
                    {
                        OnPropertyChanged("AnnotationStyle");
                    };
                }
                return _annotationStyles[this.NewAnnotationType];
            }
        }

        internal Data.AnnotationStyle GetAnnotationStyle(Data.NewAnnotationType type)
        {
            if (!_annotationStyles.Keys.Contains(type))
            {
                _annotationStyles[type] = new Data.AnnotationStyle();
                _annotationStyles[type].PropertyChanged += (o, e) =>
                {
                    OnPropertyChanged("AnnotationStyle");
                };
            }
            return _annotationStyles[type];
        }


        private Data.AnnotationStyle editingAnnotationStyle;
        public Data.AnnotationStyle EditingAnnotationStyle
        {
            get
            {
                if (editingAnnotationStyle == null)
                {
                    editingAnnotationStyle = new Data.AnnotationStyle();
                    editingAnnotationStyle.PropertyChanged += (o, e) =>
                    {
                        OnPropertyChanged("EditingAnnotationStyle");
                    };
                }
                return editingAnnotationStyle;
            }
        }
        
    }

    
}
